<?php
namespace JET_ABAF;

// If this file is called directly, abort.
if ( ! defined( 'WPINC' ) ) {
	die;
}

/**
 * Plug-in code into JetEngine
 */
class Engine_Plugin {

	private $done = false;

	public function __construct() {

		// Register field for booking form
		add_filter(
			'jet-engine/forms/booking/field-types',
			array( $this, 'register_dates_field' )
		);

		// Regsiter notification for booking form
		add_filter(
			'jet-engine/forms/booking/notification-types',
			array( $this, 'register_booking_notification' )
		);

		add_action(
			'jet-engine/forms/booking/notifications/fields-after',
			array( $this, 'notification_fields' )
		);

		add_action(
			'jet-engine/forms/edit-field/before',
			array( $this, 'edit_fields' )
		);

		// Add form field template
		add_action(
			'jet-engine/forms/booking/field-template/check_in_out',
			array( $this, 'field_template' ), 10, 3
		);

		// Register notification handler
		add_filter(
			'jet-engine/forms/booking/notification/apartment_booking',
			array( $this, 'handle_notification' ), 1, 2
		);

	}

	/**
	 * Register new dates fields
	 *
	 * @return [type] [description]
	 */
	public function register_dates_field( $fields ) {
		$fields['check_in_out'] = __( 'Check-in/check-out dates' );
		return $fields;
	}

	/**
	 * Register booking notifications
	 *
	 * @param  [type] $notifications [description]
	 * @return [type]                [description]
	 */
	public function register_booking_notification( $notifications ) {
		$notifications['apartment_booking'] = __( 'Apartment booking' );
		return $notifications;
	}

	/**
	 * Render additional edit fields
	 *
	 * @return [type] [description]
	 */
	public function edit_fields() {
		?>
		<div class="jet-form-editor__row" v-if="'check_in_out' === currentItem.settings.type">
			<div class="jet-form-editor__row-label"><?php _e( 'Layout:', 'jet-engine' ); ?></div>
			<div class="jet-form-editor__row-control">
				<select type="text" v-model="currentItem.settings.cio_field_layout">
					<option value="single"><?php
						_e( 'Single field', 'jet-engine' );
					?></option>
					<option value="separate"><?php
						_e( 'Separate fields for check in and check out dates', 'jet-engine' );
					?></option>
				</select>
			</div>
		</div>
		<div class="jet-form-editor__row" v-if="'check_in_out' === currentItem.settings.type">
			<div class="jet-form-editor__row-label"><?php _e( 'Fields position:', 'jet-engine' ); ?></div>
			<div class="jet-form-editor__row-control">
				<select type="text" v-model="currentItem.settings.cio_fields_position">
					<option value="inline"><?php
						_e( 'Inline', 'jet-engine' );
					?></option>
					<option value="list"><?php
						_e( 'List', 'jet-engine' );
					?></option>
				</select>
				<div><i>* - For separate fields layout</i></div>
			</div>
		</div>
		<div class="jet-form-editor__row" v-if="'check_in_out' === currentItem.settings.type">
			<div class="jet-form-editor__row-label"><?php _e( 'Check In field label:', 'jet-engine' ); ?></div>
			<div class="jet-form-editor__row-control">
				<input type="text" v-model="currentItem.settings.first_field_label">
				<div><i>* - if you are using separate fields for check in and check out dates,<br> you need to left default "Label" empty and use this option for field label</i></div>
			</div>
		</div>
		<div class="jet-form-editor__row" v-if="'check_in_out' === currentItem.settings.type">
			<div class="jet-form-editor__row-label"><?php _e( 'Placeholder:', 'jet-engine' ); ?></div>
			<div class="jet-form-editor__row-control">
				<input type="text" v-model="currentItem.settings.first_field_placeholder">
			</div>
		</div>
		<div class="jet-form-editor__row" v-if="'check_in_out' === currentItem.settings.type">
			<div class="jet-form-editor__row-label"><?php _e( 'Check Out field label:', 'jet-engine' ); ?></div>
			<div class="jet-form-editor__row-control">
				<input type="text" placeholder="For separate fields layout" v-model="currentItem.settings.second_field_label">
			</div>
		</div>
		<div class="jet-form-editor__row" v-if="'check_in_out' === currentItem.settings.type">
			<div class="jet-form-editor__row-label"><?php _e( 'Check Out field placeholder:', 'jet-engine' ); ?></div>
			<div class="jet-form-editor__row-control">
				<input type="text" placeholder="For separate fields layout" v-model="currentItem.settings.second_field_placeholder">
			</div>
		</div>
		<?php
	}

	/**
	 * Render additional notification fields
	 *
	 * @return [type] [description]
	 */
	public function notification_fields() {
		?>
		<div class="jet-form-editor__row" v-if="'apartment_booking' === currentItem.type">
			<div class="jet-form-editor__row-label"><?php
				_e( 'Apartment ID field:', 'jet-engine' );
			?></div>
			<div class="jet-form-editor__row-control">
				<select v-model="currentItem.booking_apartment_field">
					<option value="">--</option>
					<option v-for="field in availableFields" :value="field">{{ field }}</option>
				</select>
			</div>
		</div>
		<div class="jet-form-editor__row" v-if="'apartment_booking' === currentItem.type">
			<div class="jet-form-editor__row-label"><?php
				_e( 'Check-in/Check-out date field:', 'jet-engine' );
			?></div>
			<div class="jet-form-editor__row-control">
				<select v-model="currentItem.booking_dates_field">
					<option value="">--</option>
					<option v-for="field in availableFields" :value="field">{{ field }}</option>
				</select>
			</div>
		</div>
		<?php
		$columns = Plugin::instance()->db->get_additional_db_columns();

		if ( $columns ) {
			?>
			<div class="jet-form-editor__row" v-if="'apartment_booking' === currentItem.type">
				<div class="jet-form-editor__row-label">
					<?php _e( 'DB columns map:', 'jet-engine' ); ?>
					<div class="jet-form-editor__row-notice"><?php
						_e( 'Set <i>inserted_post_id</i> to add inserted post ID for Insert Post notification', 'jet-enegine' );
					?></div>
				</div>
				<div class="jet-form-editor__row-fields">
					<?php foreach ( $columns as $column ) {
					?>
					<div class="jet-form-editor__row-map">
						<span><?php echo $column; ?></span>
						<input type="text" v-model="currentItem.db_columns_map_<?php echo $column; ?>">
					</div>
					<?php
					} ?>
				</div>
			</div>
			<?php
		}

	}

	/**
	 * Render check-in check-out dates
	 *
	 * @return void
	 */
	public function field_template( $template, $args, $builder ) {

		$layout = ! empty( $args['cio_field_layout'] ) ? esc_attr( $args['cio_field_layout'] ) : 'single';

		if ( 'single' === $layout ) {
			$template = JET_ABAF_PATH . 'templates/form-field-single.php';
		} else {
			$template = JET_ABAF_PATH . 'templates/form-field-separate.php';
		}

		if ( ! $this->done ) {

			ob_start();
			include JET_ABAF_PATH . 'assets/js/booking-init.js';
			$init_datepicker = ob_get_clean();

			wp_register_script(
				'moment-js',
				JET_ABAF_URL . 'assets/lib/moment/js/moment.js',
				array(),
				'2.4.0',
				true
			);

			$handle = 'jquery-date-range-picker';

			wp_enqueue_script(
				$handle,
				JET_ABAF_URL . 'assets/lib/jquery-date-range-picker/js/daterangepicker.min.js',
				array( 'jquery', 'moment-js' ),
				'1.0.0',
				true
			);

			$custom_labels = Plugin::instance()->dashboard->get( 'use_custom_labels' );
			$custom_labels = filter_var( $custom_labels, FILTER_VALIDATE_BOOLEAN );

			$weekly_bookings = Plugin::instance()->dashboard->get( 'weekly_bookings' );
			$weekly_bookings = filter_var( $weekly_bookings, FILTER_VALIDATE_BOOLEAN );

			if ( ! $weekly_bookings ) {
				$one_day_bookings = Plugin::instance()->dashboard->get( 'one_day_bookings' );
				$one_day_bookings = filter_var( $one_day_bookings, FILTER_VALIDATE_BOOLEAN );
			} else {
				$one_day_bookings = false;
			}

			wp_add_inline_script( $handle, $init_datepicker );

			wp_localize_script( $handle, 'JetABAFData', array(
				'booked_dates'     => $this->get_booked_dates( $builder->post->ID ),
				'css_url'          => JET_ABAF_URL . 'assets/lib/jquery-date-range-picker/css/daterangepicker.min.css',
				'layout'           => $layout,
				'custom_labels'    => $custom_labels,
				'labels'           => Plugin::instance()->dashboard->get_labels(),
				'weekly_bookings'  => $weekly_bookings,
				'one_day_bookings' => $one_day_bookings,
			) );

			$this->done = true;

		}

		return $template;

	}

	/**
	 * Returns booked dates
	 *
	 * @return [type] [description]
	 */
	public function get_booked_dates( $post_id ) {

		$bookings  = Plugin::instance()->db->get_future_bookings( $post_id );
		$units     = Plugin::instance()->db->get_apartment_units( $post_id );
		$units_num = ! empty( $units ) ? count( $units ) : 0;
		$dates     = array();

		if ( empty( $bookings ) ) {
			return array();
		}

		$weekly_bookings = Plugin::instance()->dashboard->get( 'weekly_bookings' );
		$weekly_bookings = filter_var( $weekly_bookings, FILTER_VALIDATE_BOOLEAN );

		if ( ! $units_num || 1 === $units_num ) {
			foreach ( $bookings as $booking ) {

				$from = $booking['check_in_date'];
				$to   = $booking['check_out_date'];
				$from = new \DateTime( date( 'F d, Y', $from ) );
				$to   = new \DateTime( date( 'F d, Y', $to ) );

				$formatted_from = $from->format( 'Y-m-d' );
				$formatted_to   = $to->format( 'Y-m-d' );

				if ( $formatted_from === $formatted_to ) {
					$dates[] = $formatted_from;
					continue;
				}

				if ( $weekly_bookings ) {
					$to = $to->modify( '+1 day' );
				}

				$period = new \DatePeriod( $from, new \DateInterval( 'P1D' ), $to );

				foreach ( $period as $key => $value ) {
					$dates[] = $value->format( 'Y-m-d' );
				}

			}
		} else {

			$booked_units = array();

			foreach ( $bookings as $booking ) {

				$from = $booking['check_in_date'];
				$to   = $booking['check_out_date'];
				$from = new \DateTime( date( 'F d, Y', $from ) );
				$to   = new \DateTime( date( 'F d, Y', $to ) );

				if ( $weekly_bookings ) {
					$to = $to->modify( '+1 day' );
				}

				$period = new \DatePeriod( $from, new \DateInterval( 'P1D' ), $to );

				foreach ( $period as $key => $value ) {
					if ( empty( $booked_units[ $value->format( 'Y-m-d' ) ] ) ) {
						$booked_units[ $value->format( 'Y-m-d' ) ] = 1;
					} else {
						$booked_units[ $value->format( 'Y-m-d' ) ]++;
					}
				}
			}

			foreach ( $booked_units as $date => $booked_units_num ) {
				if ( $units_num <= $booked_units_num ) {
					$dates[] = $date;
				}
			}

		}

		return $dates;

	}

	/**
	 * Handle apartment booking notification
	 *
	 * @return [type] [description]
	 */
	public function handle_notification( $args, $notifications ) {

		if ( empty( $args['booking_dates_field'] ) ) {
			return;
		}

		if ( empty( $args['booking_apartment_field'] ) ) {
			return;
		}

		$ap_field     = $args['booking_apartment_field'];
		$dates_field  = $args['booking_dates_field'];
		$apartment_id = isset( $notifications->data[ $ap_field ] ) ? $notifications->data[ $ap_field ] : false;

		if ( isset( $notifications->data[ $dates_field . '__in' ] ) ) {

			$key_in  = $dates_field . '__in';
			$key_out = $dates_field . '__out';
			$dates   = array();

			$date_in = isset( $notifications->data[ $key_in ] ) ? $notifications->data[ $key_in ] : false;

			if ( ! empty( $notifications->data[ $key_out ] ) ) {
				$date_out = $notifications->data[ $key_out ];
			} else {
				$date_out = $date_in;
			}

			$dates = array( $date_in, $date_out );

		} else {
			$dates = isset( $notifications->data[ $dates_field ] ) ? $notifications->data[ $dates_field ] : false;
			$dates = explode( ' - ', $dates );

			if ( 1 === count( $dates ) ) {
				$dates[] = $dates[0];
			}

		}

		if ( empty( $dates ) || 2 !== count( $dates ) ) {
			/**
			 * @todo: handle error
			 */
			return;
		}

		$notifications->data['_check_in_date']  = $dates[0];
		$notifications->data['_check_out_date'] = $dates[1];

		$in  = strtotime( $dates[0] );
		$out = strtotime( $dates[1] );

		if ( $in === $out ) {
			$out = $out + 12 * HOUR_IN_SECONDS;
		}

		$booking = array(
			'apartment_id'   => $apartment_id,
			'check_in_date'  => $in,
			'check_out_date' => $out,
		);

		$columns = Plugin::instance()->db->get_additional_db_columns();

		if ( $columns ) {
			foreach ( $columns as $column ) {
				$data_key = isset( $args['db_columns_map_' . $column ] ) ? $args['db_columns_map_' . $column ] : false;

				if ( $data_key && ! empty( $notifications->data[ $data_key ] ) ) {
					$booking[ $column ] = $notifications->data[ $data_key ];
				}

			}
		}

		Plugin::instance()->db->insert_booking( $booking );

	}

}
